<?php
/*
	This PHP code is a part of EDPXY web service
	Copyright (C) 2018 DDS Central
	
	This source code is a property of DDS Central.
	Unauthorized distribution and use of this code is strictly prohibited.

	Author grants permission to use this code separately from EDPXY and Phoenix web services.

	Functions for proxying HTTP and TCP requests and processing URLs
*/


// same as str_replace, but only replaces the first occurence
function str_replace_once($needle, $replace, $haystack)
{
	$pos = strpos($haystack, $needle);
	
	if ($pos === false)
	{
		return $haystack;
	}
	
	return substr_replace($haystack, $replace, $pos, strlen($needle));
}

// returns anything after the script name, eg. http://domain.com/script.php/script/path.txt?one=two would return: /script/path.txt?one=two
function get_request_under_script()
{
	$request_uri = $_SERVER['REQUEST_URI'];
	$script_name = $_SERVER['SCRIPT_NAME'];
	$replace     = "";
	
	$result = str_replace_once($script_name, $replace, $request_uri);

	return $result;
}	

// parse HTTP header
function parse_http_header($str) 
{
	$lines = explode("\r\n", $str);
	$head  = array(array_shift($lines));
	foreach ($lines as $line)
	{
		if (empty($line))
		{
			continue;
		}
		
		list($key, $val) = explode(':', $line, 2);
		if ($key == 'Set-Cookie')
		{
			$head['Set-Cookie'][] = trim($val);
		} else {
			$head[$key] = trim($val);
		}
	}
	return $head;
}

function headers_is_chunked($headers)
{
	$chunked = isset($headers['Transfer-Encoding']) ? $headers['Transfer-Encoding'] : false;
	return ($chunked == 'chunked');
}

function headers_get_mimetype($headers)
{
	$mimetype = isset($headers['Content-Type']) ? $headers['Content-Type'] : "application/octet-stream";
	return $mimetype;
}

function headers_get_length($headers)
{
	$length = isset($headers['Content-Length']) ? intval($headers['Content-Length']) : 0;
	return $length;
}

// basic HTTP proxy for streaming or downloads.
function proxy_http_request($host, $port, $request, $timeout, $mimetype, $filename, $streamed)
{
	$ishead = ($_SERVER['REQUEST_METHOD'] == 'HEAD');
	
	// like 64 bit FF 18 on Windows 7 x64
	$user_agent = "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:18.0) Gecko/20130119 Firefox/18.0";
	
	$fp = fsockopen($host, $port, $errno, $errstr, $timeout);
	if (!$fp)
	{
		echo "$errstr ($errno)<br />";
		return false;
	}

	$out = "GET $request HTTP/1.1\r\n";
	$out .= "Connection: Close\r\n";
	$out .= "Host: $host\r\n";
	$out .= "User-Agent: $user_agent\r\n\r\n";
    
	fwrite($fp, $out);
	
	$ismimetype = !empty($mimetype);
	
	if ($ismimetype)
	{
		header("Content-Type: $mimetype");
	}
	
	if (!empty($filename))
	{
		header("Content-Disposition: attachment; filename=$filename");
	}
	
	$headers_received = false;
	$headers = "";
	$headers_response = array();

	while (!feof($fp))
	{
        // reset timer
		set_time_limit(0);
		
		if ($headers_received)
		{
			// don't send data for HEAD requests
			if ($ishead)
			{
				fclose($fp);
				return true;
			}
			
			$data = fgets($fp);
			echo $data;

			if ($streamed)
			{
				flush();
			}
		} else {
			$headers .= fgets($fp);
           
			if (strpos($headers, "\n\n") !== false)
			{
                $headers_received = true;
				$headers_response = parse_http_header($headers);
				
				// correctly proxy chunked responses
				if (headers_is_chunked($headers_response))
				{
					header("Transfer-Encoding: chunked");
				}
				
				// Use supplied mimetype if none specified
				if (!$ismimetype)
				{
					$mimetype = headers_get_mimetype($headers_response);
					header("Content-Type: $mimetype");
				}
				
				// Add content length if supplied
				$length = headers_get_length($headers_response);
				if ($length)
				{
					header("Content-Length: $length");
				}
			}
		}
	}
    
	fclose($fp);
	
	return true;		
}

// basic TCP proxy for reading data out of a socket
function proxy_tcp_request($host, $port, $timeout, $mimetype, $connect_on_head)
{
    $ishead = ($_SERVER['REQUEST_METHOD'] == 'HEAD');
    
    $a_mimetype = !empty($mimetype) ? $mimetype : 'application/octet-stream';
	
    // do we have a HEAD request ? if so, do we just return Content-Type or proceed with the actual connection ?
    if ($ishead && !$connect_on_head)
    {
        header("Content-Type: $a_mimetype");
        
        return true;
    }
    
    $socket = fsockopen($host, $port, $errno, $errstr, $timeout);

    if (!$socket)
    {
        return false;
    }
    
    header("Content-Type: $a_mimetype");
    
    // if we get here with a HEAD request, close the connection and return success
    if ($ishead)
    {
        fclose($socket);
        
        return true;
    }
    
    while (!feof($socket))
    {
        $data = fgets($socket);
        echo $data;
  
        flush();
    }

    fclose($socket);
    
    return true;
}

?>
